/*
 * Copyright (C) 2019 CZ.NIC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* ========================================================================= */
/* SWIG setting and definitions. */
/* ========================================================================= */

/* Let SWIG generate the default constructor. */
%nodefaultdtor isds_event;

/* Manually added properties into Python proxy class. */
%feature ("docstring") isds_event::time "Event time.

:note: A copy of the value is returned when read. A copy of the value is stored when written.

:type: :class:`isds_time`
"
%attribute_custom_nodisown_(isds_event, struct timeval *, time,
    get_time, set_time,
    _isds_event_get_time(self_), _isds_event_set_time(self_, val_));
%ignore _isds_event_get_time;
%ignore _isds_event_set_time;

%delobject _isds_event_free;
%ignore isds_event_free;
#ifdef ISDS_DEBUG
%inline
%{
	void _isds_event_free(struct isds_event *ev)
	{
		fprintf(stderr, "******** ISDS_EVENT free 0x%" PRIXPTR " ********\n", (uintptr_t)ev);
		if (NULL != ev) {
			isds_event_free(&ev);
		}
	}
%}
#else /* !ISDS_DEBUG */
%inline
%{
	void _isds_event_free(struct isds_event *ev)
	{
		if (NULL != ev) {
			isds_event_free(&ev);
		}
	}
%}
#endif /* ISDS_DEBUG */

%newobject _isds_event_get_time;
%inline
%{
	struct timeval *_isds_event_get_time(const struct isds_event *ev)
	{
		if (NULL != ev) {
			struct timeval *ret = NULL;
			ret = _timeval_copy(ev->time);
			return ret;
		}
		return NULL;
	}

	void _isds_event_set_time(struct isds_event *ev, const struct timeval *tv)
	{
		if (NULL == ev) {
			return;
		}
		if (NULL != ev->time) {
			free(ev->time);
		}
		ev->time = _timeval_copy(tv);
	}
%}

/* ========================================================================= */
/* Debugging related code. */
/* ========================================================================= */

/* None. */

/* ========================================================================= */
/* Added C code. */
/* ========================================================================= */

/* None */

/* ========================================================================= */
/* Encapsulating Python code. */
/* ========================================================================= */

%feature("docstring") isds_event "ISDS event.

This class holds the event content.
"

%feature("docstring") isds_event::type "Type of the event.

:type: :ref:`isds_event_type`
"

%feature("docstring") isds_event::description "Human-readable event description generated by ISDS (usually in Czech).

:type: string
"

%extend isds_event {
	%pythoncode
	%{
		__swig_destroy__ = _isds._isds_event_free
	%}
}
