#!/usr/bin/env python3
# vim:fileencoding=utf-8
#
# Message functionality.
#

import isds
import unittest

# Global message.
msg = isds.isds_message()

class TestIsdsMessage(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestIsdsMessage, self).__init__(*args, **kwargs) # Call the parrents __init__ method.
        #self.msg = isds.isds_message()
        self.msg = None

    def test_raw(self):
        self.msg = msg
        val = bytearray(b"abcdef")
        self.assertEqual(self.msg.raw, bytearray(b""), "Unexpected raw value.")
        self.msg.raw = val
        self.assertEqual(self.msg.raw, val, "Unexpected raw value.")
        self.msg.raw = None
        self.assertEqual(self.msg.raw, bytearray(b""), "Unexpected raw value.")
        self.msg.raw = val
        self.assertEqual(self.msg.raw, val, "Unexpected raw value.")

    def test_raw_type(self):
        self.msg = msg
        val = isds.RAWTYPE_PLAIN_SIGNED_INCOMING_MESSAGE
        self.assertEqual(self.msg.raw_type, isds.RAWTYPE_INCOMING_MESSAGE, "Unexpected raw_type value.")
        self.msg.raw_type = val
        self.assertEqual(self.msg.raw_type, val, "Unexpected raw_type value.")
        self.msg.raw_type = isds.RAWTYPE_INCOMING_MESSAGE
        self.assertEqual(self.msg.raw_type, isds.RAWTYPE_INCOMING_MESSAGE, "Unexpected raw_type value.")
        self.msg.raw_type = val
        self.assertEqual(self.msg.raw_type, val, "Unexpected raw_type value.")

    def test_envelope(self):
        self.msg = msg
        val = isds.isds_envelope()
        val.dmID = "1234"
        self.assertEqual(self.msg.envelope, None, "Unexpected envelope value.")
        self.msg.envelope = val
        self.assertEqual(self.msg.envelope.dmID, val.dmID, "Unexpected envelope value.")
        self.msg.envelope = None
        self.assertEqual(self.msg.envelope, None, "Unexpected envelope value.")
        self.msg.envelope = val
        self.assertEqual(self.msg.envelope.dmID, val.dmID, "Unexpected envelope value.")

    def test_documents(self):
        self.msg = msg
        val = []
        document = isds.isds_document()
        val.append(document)
        self.assertEqual(self.msg.documents, [], "Unexpected documents value.")
        self.msg.documents = val
        self.assertEqual(len(self.msg.documents), 1, "Unexpected documents value.")
        self.msg.documents = []
        self.assertEqual(self.msg.documents, [], "Unexpected documents value.")
        self.msg.documents = val
        self.assertEqual(len(self.msg.documents), 1, "Unexpected documents value.")
